#!/usr/bin/perl

# __VERSION__ 1.0

# cwgd2ovl.pl

# This script converts a sorted .cwgd file to a map overlay for the mapping-software
# of the German "Landesvermessungsamt" (state geodatic office) Northrine-Westfalia.
#
# written by: Stefan Gofferje
# Modified by Mike Kershaw
#
# License: GPL
#
# usage: cwgd2ovl.pl <input file> <output file>

if ($ARGV[0] eq "") {
    print "No cwgd file given.\n";
    print "Usage: cwgd2ovl <input GPS file> <output file>\n";
    exit(1);
}
my $file = $ARGV[0];

my $outfile = "$file.cwgd";
if ($ARGV[1] eq "") {
    print "No output file given, assuming $file.cwgd\n";
} else {
    $outfile = $ARGV[1];
}

#--------------------------------------------------------------------
#Search regexp
#--------------------------------------------------------------------

#my $pat = '^(..:..:..:..:..:..),(.+),(\d+\.\d+),(\d+\.\d+),(\d+)';
# BSSID SSID LAT LON ALT SPEED FIX QUALITY POWER NOISE TIME
my $pat = "^(..:..:..:..:..:..)\t(.+)\t(.+)\t(.+)\t(.+)\t(.+)\t(.+)\t(.+)\t(.+)\t(.+)\t(.+)";

# Open file
open(CFILE, "$file") or die "Unable to open input file $file";
open(OFILE, ">$outfile") or die "Unable to open output file $file";

#--------------------------------------------------------------------
#Chop line into interesting parts
#--------------------------------------------------------------------

$linecount = "1";
$oldx = " ";
$oldy = " ";
$oldmac = " ";

while ($line=<CFILE>)
{
  $_ = $line;
  $result = m/$pat/o;

  $north = $3;
  $east = $4;
  $ssid = $2;
  $mac = $1;
  $snr = $8;

  # Iterate over track records
  if ($1 eq "00:00:00:00:00:00") {
	continue;
  }

  if ( ($result eq "1") && ($east > 0) && ($north > 0) )
  {
    if ( ($oldx ne $east) && ($oldy ne $north) )
    {
      $oldx = $east;
      $oldy = $north;
      print "[Symbol $linecount]\n";
      print "Typ=6\n";
      print "Group=1\n";
      print "Width=1\n";
      print "Height=1\n";
      print "Dir=100\n";
      if ($snr < 5) { print "Col=1\n"; }
      if ( ($snr >= 5) && ($snr < 17) ) { print "Col=4\n"; }
      if ($snr >= 17) { print "Col=2\n"; }
      print "Zoom=1\n";
      print "Size=103\n";
      print "Area=1\n";
      print "XKoord=$east\n";
      print "YKoord=$north\n";
      $linecount = $linecount + 1;
    }
  }
} 
$linecount = $linecount - 1;
print "[Overlay]\n";
print "symbols=$linecount\n";

#--------------------------------------------------------------------
#End
#--------------------------------------------------------------------
