#!/usr/bin/perl

# __VERSION__ 1.0

# kismet2cwgd.pl
#
# Kismet GPS to Common Wireless GPS Data format.
#
# Mike Kershaw
#
# usage: kismet2cwgd.pl <input file> <output file>

if ($ARGV[0] eq "") {
	print "No GPS file given.\n";
	print "Usage: kismet2cwgd <input GPS file> <output file>\n";
	exit(1);
}
my $file = $ARGV[0];

my $outfile = "$file.cwgd";
if ($ARGV[1] eq "") {
	print "No output file given, assuming $file.cwgd\n";
} else {
	$outfile = $ARGV[1];
}

my $SSID_SIZE = 32;

my $fileheader = "LCll";
my $fileheadersize = "16";

# netheader 3 and 4
my $netheader = "sCCCCCCA".$SSID_SIZE;
my $netheadersize = "40";

# Netheader 1 and 2
my $netheader1 = "CCCCCCCA".$SSID_SIZE;
my $netheadersize1 = "39";

# data packet 4
#num: 0 lat: 2 lat_mant: 4 lon: 12 lon_mant: 16 alt: 24 alt_mant: 28 spd: 36 spd_mant: 40 fix: 48 quality: 50 power: 52 noise: 54 ts_sec: 56 ts_usec: 60
my $datapkt = "SsLLsxxLLsxxLLsxxLLsSSSll";
my $datapktsize = "64";

# Data packet 2 and 3
# num: 0 lat: 2 lat_mant: 4 lon: 12 lon_mant: 16 alt: 24 alt_mant: 28 spd: 36 spd_mant: 40 fix: 48 quality: 50 power: 52 noise: 56
my $datapkt3 = "CxsLLsxxLLsxxLLsxxLLSll";
my $datapktsize3 = "60";

# Data packet 1
my $datapkt1 = "CxsLLsxxLLsxxLLsxxLLll";
my $datapktsize1 = "60";

print("Opening gps file: ". $file ."\n");

open(GPSFILE, $file) or die "Could not open $file";
open(OUTFILE, ">$outfile") or die "Could not open $outfile";

binmode GPSFILE;

sysread(GPSFILE, $data, $fileheadersize);

($magic, $version, $start_sec, $start_usec) = unpack($fileheader, $data);

if ($magic == 0xDEADBEEF) {
	print("Got magic, continuing\n");
}

if ($version > 4 or $version < 1) {
	printf("Got an unknown version %d\n", $version);
	exit(1);
}

printf("Got GPS file version %d\n", $version);

if ($version == 1 or $version == 2) {
	$cnt = 0;
	do {
		sysread(GPSFILE, $data, $netheadersize1);
		($netnum, $bssid[0], $bssid[1], $bssid[2], $bssid[3], $bssid[4], $bssid[5], $ssid) = 
			unpack($netheader1, $data);

		$bssid = sprintf("%02X:%02X:%02X:%02X:%02X:%02X",
				$bssid[0], $bssid[1], $bssid[2], $bssid[3], $bssid[4], $bssid[5]);

		@netnums[$netnum] = ($bssid, $ssid);

	} while ($cnt++ < 253);
} elsif ($version == 4 or $version == 3) {
	$cnt = 0;
	do {
		sysread(GPSFILE, $data, $netheadersize);
		($netnum, $bssid[0], $bssid[1], $bssid[2], $bssid[3], $bssid[4], $bssid[5], $ssid) = 
			unpack($netheader, $data);

		$bssid = sprintf("%02X:%02X:%02X:%02X:%02X:%02X",
				$bssid[0], $bssid[1], $bssid[2], $bssid[3], $bssid[4], $bssid[5]);

		$ssid =~ s/\t/ /g;

		@bssids[$netnum] = $bssid;
		@ssids[$netnum] = $ssid;

	} while ($cnt++ < 4095);
}

print OUTFILE "# COMMON WIRELESS GPS DATA\n";
print OUTFILE "# File format 1.0\n";
print OUTFILE "# Generated by kismet2cwgd\n\n";
print OUTFILE "# BSSID SSID LAT LON ALT SPEED FIX QUALITY POWER NOISE TIME\n\n";

print "Converting coordinates...\n";

if ($version == 1) {
	$size = $datapktsize1;
} elsif ($version == 2 or $version == 3) {
	$size = $datapktsize3;
} elsif ($version == 4) {
	$size = $datapktsize;
}

while (sysread(GPSFILE, $data, $size)) { 
	$number = $ssid = $bssid = $track = $lat = $lon = $alt = $spd = $fix = $quality = $power = $noise = -1;

	if ($version == 1) {
		($number, 
		$lat, $lat_mant1, $lat_mant2, 
		$lon, $lon_mant1, $lon_mant2, 
		$alt, $alt_mant1, $alt_mant2,
		$spd, $spd_mant1, $spd_mant2,
		$fix, 
		$ts_sec, $ts_usec) = unpack($datapkt1, $data);

		$track = 1 if $number == 256;
	} elsif ($version == 2 or $version == 3) {
		($number,
		$lat, $lat_mant1, $lat_mant2, 
		$lon, $lon_mant1, $lon_mant2, 
		$alt, $alt_mant1, $alt_mant2,
		$spd, $spd_mant1, $spd_mant2,
		$fix, 
		$power, 
		$ts_sec, $ts_usec) = unpack($datapkt3, $data);

		$track = 1 if $number == 256;
	} elsif ($version == 4) {
		($number, 
			$lat, $lat_mant1, $lat_mant2, 
			$lon, $lon_mant1, $lon_mant2, 
			$alt, $alt_mant1, $alt_mant2,
			$spd, $spd_mant1, $spd_mant2,
			$fix, 
			$quality, $power, $noise, 
			$ts_sec, $ts_usec) = unpack($datapkt, $data);

		$track = 1 if $number == 4096;
	}

	$latmant = ($lat_mant1 << 32) + $lat_mant2;
	$lonmant = ($lon_mant1 << 32) + $lon_mant2;
	$altmant = ($alt_mant1 << 32) + $alt_mant2;
	$spdmant = ($spd_mant1 << 32) + $spd_mant2;

	if ($track == -1) {
		$bssid = $bssids[$number];
		$ssid = $ssids[$number];
	} else {
		$bssid = "00:00:00:00:00:00";
		$ssid = "__TRACK__";
	}

	print OUTFILE "$bssid\t$ssid\t$lat.$latmant\t$lon.$lonmant\t$alt.$altmant\t$spd.$spdmant\t$fix\t$quality\t$power\t$noise\t". localtime($ts_sec) ."\n";


}

