#ifndef __PANELFRONT_H__
#define __PANELFRONT_H__

#include "config.h"

#if (defined(HAVE_LIBNCURSES) && defined(HAVE_LIBPANEL) && defined(BUILD_PANEL))

#ifdef HAVE_LIBCURSES
#include <curses.h>
#include <panel.h>
#else
#include <ncurses.h>
#include <panel.h>
#endif
#undef erase
#undef clear
#undef move

#include <stdio.h>
#include <time.h>
#include <list>
#include <map>
#include <string>

#include "packet.h"
#include "frontend.h"
#include "cursesfront.h"
#include "networksort.h"

// These are in the kismet_curses.cc main code
extern int sound;
extern int speech;

class PanelFront : public Frontend {
public:
    PanelFront();

    int ParseArgs(int argc, char *argv[]) { return 0; }

    int Poll();

    int InitDisplay(int in_decay, time_t in_start);

    // Draw the screen
    int DrawDisplay();

    // End
    int EndDisplay();

    int WriteStatus(string status);

    // I don't like doing this this way
    void SetColumns(string in_columns);

protected:
    typedef void (PanelFront::*panel_printer)(PANEL *);

    void RescaleDisplay();

    //void DelOldest(wireless_network *exclude);
    void ZoomNetworks();
    void SpawnPopup(string in_title, panel_printer in_print, int in_x = -1, int in_y = -1);

    void PrintKismetHelp(PANEL *in_panel);
    void PrintKismetSort(PANEL *in_panel);
    void PrintKismetPower(PANEL *in_panel);
    void PrintKismetInfo(PANEL *in_panel);
    void PrintKismetDump(PANEL *in_panel);
    void PrintKismetGroupname(PANEL *in_panel);

    void Details2Vector(wireless_network *in_net);

    string NetLine(wireless_network *net, const char *name, int sub, int group, int expanded, int tagged);

    int sub_sort;
    int sub_dump;
    int clear_dump;

    sort_type sortby;
    unsigned int display_start;
    unsigned int display_end;
    unsigned int selected;
    unsigned int cutoff;

    int print_width;
    int max_display;

    vector<string> column_vec;

    int quality, power, noise;

    int zoomed;
    string popup_title;

    display_network *details_network;
    vector<display_network *> last_displayed;

//    wireless_network *details_network;
//    vector<wireless_network *> last_displayed;

    PANEL *cur_panel;
    PANEL *popup_panel;
    panel_printer printer;

    PANEL *network_panel, *status_panel, *info_panel, *details_panel;

    WINDOW *netwin, *infowin, *statuswin, *detailswin, *popupwin;

    unsigned int popup_start, popup_end, popup_length;

    int hsize, vsize;

    int popup_paused;

    // Map of bssid's to display groups
    map<string, display_network *> bssidgroup_map;

    // Map of BSSID's to custom names
    map<string, string> bssidname_map;

    int old_sound;
    int old_speech;
    int muted;

};

#endif

#endif
